﻿using System;

using Android.App;
using Android.Content.PM;
using Android.Runtime;
using Android.Views;
using Android.Widget;
using Android.OS;
using Android.Content;
using Android.Support.V4.Content;
using Android.Support.V4.App;
using Android;
using Android.Provider;
using System.IO;
using Xamarin.Forms;
using LoadFromGallerySample.Droid;
using ZXing.Common;
using ZXing;
using System.Collections.Generic;

[assembly: Dependency(typeof(LoadImageAndroid))]
namespace LoadFromGallerySample.Droid
{
    [Activity(Label = "LoadFromGallerySample", Icon = "@mipmap/icon", Theme = "@style/MainTheme", MainLauncher = true, ConfigurationChanges = ConfigChanges.ScreenSize | ConfigChanges.Orientation)]
    public class MainActivity : global::Xamarin.Forms.Platform.Android.FormsAppCompatActivity
    {
        protected override void OnCreate(Bundle savedInstanceState)
        {
            TabLayoutResource = Resource.Layout.Tabbar;
            ToolbarResource = Resource.Layout.Toolbar;

            base.OnCreate(savedInstanceState);
            global::Xamarin.Forms.Forms.Init(this, savedInstanceState);
            LoadApplication(new App());
        }
        public event EventHandler<ActivityResultEventArgs> ActivityResult;

        protected override void OnActivityResult(int requestCode, Android.App.Result resultCode, Intent data)
        {
            if (resultCode == Android.App.Result.Ok)
            {
                ActivityResult?.Invoke(this, new ActivityResultEventArgs { Intent = data });
            }
        }

        private int PERMISSION_REQUEST_CODE = 1;

        /// <summary>
        /// Check whether this application has permission to access the external storage
        /// </summary>
        public bool PermissionGrantedForExternalStorage
        {
            get
            {
                Permission permissionResult = ContextCompat.CheckSelfPermission(this, Manifest.Permission.WriteExternalStorage);
                if (permissionResult == Permission.Granted)
                {
                    // if permission is already granted return true otherwise return false
                    return true;
                }
                else
                {
                    return false;
                }
            }
        }

        /// <summary>
        /// Request to enable permission to write the files on external storage of android device
        /// </summary>
        public void RequestPermission()
        {
            if (!ActivityCompat.ShouldShowRequestPermissionRationale(this, Manifest.Permission.WriteExternalStorage))
            {
                ActivityCompat.RequestPermissions(this, new string[] { Manifest.Permission.WriteExternalStorage }, PERMISSION_REQUEST_CODE);
            }
        }

        public override void OnRequestPermissionsResult(int requestCode, string[] permissions, Permission[] grantResults)
        {
            base.OnRequestPermissionsResult(requestCode, permissions, grantResults);
        }
    }

    public class ActivityResultEventArgs : EventArgs
    {
        public Intent Intent
        {
            get;
            set;
        }
    }

    public class LoadImageAndroid : ILoad
    {
        private static int SELECT_FROM_GALLERY = 0;
        private static int SELECT_FROM_CAMERA = 1;
        static Intent mainIntent;
        private Android.Net.Uri mImageCaptureUri;
        MainActivity activity;
        bool isCamera = false;
        private MainPage page;
        Stream stream = new MemoryStream(); 


        /// <summary>
        /// Upload image from galerry 
        /// </summary>
        /// <param name="editor"></param>
        public void UploadFromGallery(MainPage editor)
        {
            isCamera = false;
            page = editor;
            
            activity = Forms.Context as MainActivity;

            bool value = activity.PermissionGrantedForExternalStorage;
            if (!value)
            {
                activity.RequestPermission();
            }
            if (activity.PermissionGrantedForExternalStorage)
            {
                activity.ActivityResult -= LoadImage;
                activity.ActivityResult += LoadImage;
                activity.Intent = new Intent();
                activity.Intent.SetType("image/*");
                activity.Intent.SetAction(Intent.ActionGetContent);
                activity.StartActivityForResult(Intent.CreateChooser(activity.Intent, "Select Picture"), SELECT_FROM_GALLERY);
            }
        }
      
        void LoadImage(object sender, ActivityResultEventArgs e)
        {
            if (!isCamera)
            {
                var imagePath = GetPathToImage(e.Intent.Data);
                stream = System.IO.File.OpenRead(imagePath);
                page.SwitchView(stream);
            }
            else
            {
                mainIntent.PutExtra("image-path", mImageCaptureUri.Path);
                mainIntent.PutExtra("scale", true);
                stream = System.IO.File.OpenRead(mImageCaptureUri.Path);
                page.SwitchView(stream);
            }
        }

        private string GetPathToImage(Android.Net.Uri uri)
        {
            string imgId = "";
            string[] proj = { MediaStore.Images.Media.InterfaceConsts.Data };
            using (var c1 = activity.ContentResolver.Query(uri, null, null, null, null))
            {
                try
                {
                    if (c1 == null) return "";
                    c1.MoveToFirst();
                    string imageId = c1.GetString(0);
                    imgId = imageId.Substring(imageId.LastIndexOf(":") + 1);
                }
                catch (System.Exception e)
                {
                    Toast.MakeText(Forms.Context, "Unable To Load Image", ToastLength.Short);
                }
            }

            string path = null;

            string selection = MediaStore.Images.Media.InterfaceConsts.Id + " =? ";
            
            using (var cursor = activity.ContentResolver.Query(MediaStore.Images.Media.ExternalContentUri, null, selection, new string[] { imgId }, null))
            {
                try
                {
                    if (cursor == null) return path;
                    var columnIndex = cursor.GetColumnIndexOrThrow(MediaStore.Images.Media.InterfaceConsts.Data);
                    cursor.MoveToFirst();
                    path = cursor.GetString(columnIndex);
                }
                catch (System.Exception e)
                {
                    Toast.MakeText(Forms.Context, "Unable To Load Image", ToastLength.Short);
                    return "";
                }
            }
            return path;

        }

        public BinaryBitmap GetBinaryBitmap(byte[] image)
        {
            Android.Graphics.Bitmap bitmap = Android.Graphics.BitmapFactory.DecodeByteArray(image, 0, image.Length);
            byte[] rgbBytes = GetRgbBytes(bitmap);                                                                                                                                                                                                                                                                                                           
            var bin = new HybridBinarizer(new RGBLuminanceSource(rgbBytes, bitmap.Width, bitmap.Height));
            var i = new BinaryBitmap(bin);

            return i;
        }

        private byte[] GetRgbBytes(Android.Graphics.Bitmap image)
        {
            var rgbBytes = new List<byte>();
            for (int y = 0; y < image.Height; y++)
            {
                for (int x = 0; x < image.Width; x++)
                {
                    var c = new Android.Graphics.Color(image.GetPixel(x, y));

                    rgbBytes.AddRange(new[] { c.R, c.G, c.B });
                }
            }
            return rgbBytes.ToArray();
        }
    }
}